import { p$p } from './parse';
/** kebab regex */
const kr = /[A-Z\u00C0-\u00D6\u00D8-\u00DE]/g;
/** kebab case */
function kc(pattern) {
    return pattern.replace(kr, match => '-' + match.toLowerCase());
}
/** upwrap property */
/** TODO: refine it */
function up(prop, indent = '  ') {
    if (typeof prop === 'object') {
        return (' {\n' +
            Object.entries(prop).map(v => {
                return indent + `  ${kc(v[0])}: ${v[1]};`;
            }).join('\n') +
            '\n' + indent + '}');
    }
    return `: ${String(prop)};`;
}
/** create style */
function cs(selector, props, instance) {
    if (props === null)
        return null;
    const propertyNames = Object.keys(props);
    if (propertyNames.length === 0) {
        if (instance.config.preserveEmptyBlock)
            return selector + ' {}';
        return null;
    }
    const statements = [
        selector + ' {'
    ];
    propertyNames.forEach(propertyName => {
        const property = props[propertyName];
        propertyName = kc(propertyName);
        if (property !== undefined) {
            statements.push(`  ${propertyName}${up(property)}`);
        }
    });
    statements.push('}');
    return statements.join('\n');
}
/** traverse */
function t(node, selectorPaths, styles, instance) {
    if (typeof node.$ === 'string') {
        selectorPaths.push(node.$);
    }
    else if (node.$.$ !== undefined) {
        if (node.$.before !== undefined)
            node.$.before(instance.context);
        selectorPaths.push(node.$.$(instance.context));
    }
    const selector = p$p(selectorPaths, instance);
    const style = cs(selector, node.props, instance);
    if (style !== null)
        styles.push(style);
    if (node.children !== null) {
        node.children.forEach(childNode => {
            t(childNode, selectorPaths, styles, instance);
        });
    }
    selectorPaths.pop();
    if (!(typeof node.$ === 'string') && node.$.after !== undefined)
        node.$.after(instance.context);
}
export function render(node, instance) {
    const styles = [];
    t(node, [], styles, instance);
    return styles.join('\n\n');
}
